/*!
  \file   batwing.c
  \brief  The main file for the console batwing project

  A program to get posterior trees for genealogies
 
  \author Ian Wilson
  \date   2004-06-09
*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "modeltime.h"
#include "node.h"
#include "tree.h"
#include "random.h"
#include "nrutil.h"
#include "metro.h"
#include "lhood.h"
#include "cutjoin.h"
#include "pars.h"
#include "cutil.h"
#include "newick.h"
#ifdef CHECK
#include "check.h"
#endif
#include "missing.h"
#include "myio.h"


/**************************************************************************/
/*  The Main Function  - Batwing (infile) (outfile) (seed)                */
/**************************************************************************/
int main(int argc, char *argv[])
{
  int i,j,pic_count=1;
  char picname[40],picstr[30],*inputfile="",*outputfile="",*seed="";
  const char *message;
  tree t;
  parameters p;
  FILE *OUTFILE, *PARS, *CONSENSUSP,*CONSENSUST;
	
  if (argc==1) {
    fprintf(stderr,"no arguments entered:\n");
    inputfile=getstringfromkeyboard("input file","infile");
  } else {
    inputfile=argv[1];
    if (argc>=3) outputfile=argv[2];
    if (argc==4) seed=argv[3];
  }
  
  if (argc>4) {
    fprintf(stderr,"use:\n%s infile (outfile) (seed)\n",argv[0]);  
    exit(-1);
  }

  p=setup_parameters(inputfile,outputfile,seed,quiet); 

  strcpy(picname,p.outfilename);
  strcpy(picstr,p.outfilename);
  if ((message=check_parsb(&p,quiet))!=NULL) {
    fprintf(stderr,"%s\n",message);	exit(1);
  }

  if (p.initialfilename!=NULL) {

    t=read_tree(p.initialfilename,p.constsites);
    if (p.seed!=-1) {
      fprintf(stderr,"new seed set to %d\n",p.seed);
      starTup(p.seed);	
    } else {
      fprintf(stderr,"starting from old random seed\n");
    }
    PARS = openstemleafoutputfile(picname,"par");
  } else {
    read_parameters_data(&p); 
    t = partreestartup(&p);
    printpardetails(&p,stdout,1);
    PARS = openstemleafoutputfile(picname,"par");
    printpardetails(&p,PARS,0);	
  } 

  fprintf(stderr,"starting up\n");
  OUTFILE=openoutputfile(picname);
  output_names(OUTFILE,&t,&p);
  strcat(picname,".iit");
  printalltreeinfo(picname,&t,&p);
  if (p.tconsensus)  CONSENSUST=openstemleafoutputfile(picstr,"tcns");
  if (p.pconsensus)  CONSENSUSP=openstemleafoutputfile(picstr,"pcns");
#ifdef CHECK
  checktree(&t,"before start");
#endif
  for (i=1;i<=p.reps;i++) {
    for (j=1 ;j<=p.Nbetsamp;j++) metro_step(&t,p.treebetN,p.move_nodes);	
    output_line(OUTFILE,&t,&p);
    if (p.pconsensus) write_shapeNewickpoptree(CONSENSUSP,&t.populationtree);
    if (p.tconsensus) write_Newickshape(t.root,t.sample,CONSENSUST);
		
    if (i%20==0 ) {
      if (i%1000==0) fprintf(stderr,".%d\n",i); 
      else fprintf(stderr,"."); 
      fflush(stderr);
    }
    if (i%p.picgap==0) {
      get_counter_filename(picname,picstr,pic_count++);
      printalltreeinfo(picname,&t,&p);
    }
  } 
  strcpy(picname,picstr);strcat(picname,".end");
  printalltreeinfo(picname,&t,&p);   
  rescale_proportions(&t,p.reps,p.treebetN,p.Nbetsamp);
  print_proportions(stdout,&t);
  print_proportions(PARS,&t);
  if (p.tconsensus) fclose(CONSENSUST);
  if (p.pconsensus) fclose(CONSENSUSP);

  fclose(OUTFILE);
  fclose(PARS);
  destroy_tree(&t);
  return 1;
}
/**************************************************************************/


